package oracle.iam.sample.notification;

import static oracle.iam.identity.rolemgmt.api.RoleManagerConstants.ROLE_KEY_PARAM;
import static oracle.iam.identity.rolemgmt.api.RoleManagerConstants.USER_KEYS_PARAM;
import static oracle.iam.identity.rolemgmt.api.RoleManagerConstants.USER_KEY_PARAM;
import static oracle.iam.identity.rolemgmt.utils.RoleManagerUtils.getParameterValue;
import static oracle.iam.identity.usermgmt.api.UserManagerConstants.AttributeName.MANAGER_KEY;
import static oracle.iam.identity.usermgmt.api.UserManagerConstants.AttributeName.USER_LOGIN;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import oracle.iam.identity.exception.NoSuchUserException;
import oracle.iam.identity.exception.UserLookupException;
import oracle.iam.identity.usermgmt.api.UserManager;
import oracle.iam.identity.usermgmt.vo.User;
import oracle.iam.notification.api.NotificationService;
import oracle.iam.notification.exception.EventException;
import oracle.iam.notification.exception.MultipleTemplateException;
import oracle.iam.notification.exception.NotificationException;
import oracle.iam.notification.exception.NotificationResolverNotFoundException;
import oracle.iam.notification.exception.TemplateNotFoundException;
import oracle.iam.notification.exception.UnresolvedNotificationDataException;
import oracle.iam.notification.exception.UserDetailsNotFoundException;
import oracle.iam.notification.vo.NotificationEvent;
import oracle.iam.platform.Platform;
import oracle.iam.platform.authz.exception.AccessDeniedException;
import oracle.iam.platform.kernel.EventFailedException;
import oracle.iam.platform.kernel.spi.PostProcessHandler;
import oracle.iam.platform.kernel.vo.AbstractGenericOrchestration;
import oracle.iam.platform.kernel.vo.BulkEventResult;
import oracle.iam.platform.kernel.vo.BulkOrchestration;
import oracle.iam.platform.kernel.vo.EventResult;
import oracle.iam.platform.kernel.vo.Orchestration;

public class RoleGrantNotificationEvent implements PostProcessHandler {
	
	// This custom post process event handler sends notification to 
	// user/manager when a role is granted to the user.
	
	/* (non-Javadoc)
	 * @see oracle.iam.platform.kernel.spi.PostProcessHandler#execute(long, 
	 * 	long, oracle.iam.platform.kernel.vo.BulkOrchestration)
	 */
	public BulkEventResult execute(long processId, long eventId, 
		BulkOrchestration orchestration) {
		System.out.println("Inside post handler" );
		System.out.println("Inside post handler -- Operation Identified" );
		String userKey=null;
		
		//Retrieving role's key assigned to the user 
		String roleKey = (String)getParameterValue(orchestration, ROLE_KEY_PARAM);
		
		//Workaround to find userkey as this operation is being assumed as bulk 
		// orchestration and hence user key is stored as arrayList
		Object userObj = getParameterValue(orchestration, USER_KEYS_PARAM);
		if (userObj == null) {
			userObj = getParameterValue(orchestration, USER_KEY_PARAM);
			if (userObj != null) {
				userKey = (String) userObj;
			}
		} else {
			userKey = ((ArrayList<String>)userObj).get(0);
		}
		
		try {
			System.out.println("Creating event and sending notification" );
			NotificationEvent event = getRoleGrantEvent(userKey, roleKey, null);
			sendNotification(event);
		} catch(Exception e) {
			throw new EventFailedException("Error occurred ", e);
		}
		return new BulkEventResult();
	}

	@Override
	public boolean cancel(long arg0, long arg1,
		AbstractGenericOrchestration arg2) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void compensate(long arg0, long arg1,
		AbstractGenericOrchestration arg2) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public EventResult execute(long arg0, long arg1, Orchestration arg2) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void initialize(HashMap<String, String> arg0) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * This method creates/configure event object corresponding to the event when 
	 * a role is assigned to a user
	 * @param userKey
	 * @param roleKey
	 * @param senderId
	 * @return {@link NotificationEvent}
	 * @throws NoSuchUserException
	 * @throws UserLookupException
	 * @throws AccessDeniedException
	 */
	private NotificationEvent getRoleGrantEvent(String userKey, String roleKey, 
		String senderId) throws NoSuchUserException, UserLookupException, 
		AccessDeniedException {
		NotificationEvent event = new NotificationEvent();
		
		//get user IDs to whom notification is to be sent and set it in the 
		//event object being created
		String[] receiverUserIds= getRecipientUserIds(userKey);	      
		event.setUserIds(receiverUserIds);
	    
		//Set template name to be used to send notification for this event
		event.setTemplateName("RoleGrantedNotificationTemplate");
	    
		//Setting senderId as null here and hence default sender ID would 
		//get picked up
		event.setSender(senderId);

		//Create a map with key value pair for the parameters declared at time 
		//of configuring notification event
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("usr_key", userKey);
		map.put("role_key", roleKey);
		event.setParams(map);
	    
		return event;
	}

	/**
	 * This method creates a string array containing list of user IDs to whom notification 
	 * is to be sent
	 * 
	 * @param userKey
	 * @return String Array containing list of user IDs to whom notification is to be sent
	 * @throws NoSuchUserException
	 * @throws UserLookupException
	 * @throws AccessDeniedException
	 */
	private String[] getRecipientUserIds(String userKey) throws NoSuchUserException, 
		UserLookupException, AccessDeniedException {

		UserManager usrMgr = Platform.getService(UserManager.class);
		
		User user = null;
		String userId = null;
		Set<String> userRetAttrs = new HashSet<String>();
	    
		//Sending notification to both the user and his/her manager 
		userRetAttrs.add(MANAGER_KEY.getId());
		userRetAttrs.add(USER_LOGIN.getId());
	    
		User manager = null;
		String managerId = null;
		String managerKey = null;
		Set<String> managerRetAttrs = new HashSet<String>();
		managerRetAttrs.add(USER_LOGIN.getId());
	    
		//Retrieving User ID
		user = usrMgr.getDetails(userKey, userRetAttrs, false);
		userId = user.getAttribute(USER_LOGIN.getId()).toString();
        
		List<String> userIds = new ArrayList<String>();
		userIds.add(userId);
        
		if (user.getAttribute(MANAGER_KEY.getId()) != null) {
			managerKey = user.getAttribute(MANAGER_KEY.getId()).toString();
			manager = usrMgr.getDetails(managerKey, managerRetAttrs, false);

			//Retrieving User's Manager ID
			managerId = manager.getAttribute(USER_LOGIN.getId()).toString();
			userIds.add(managerId);
		}
        
		//To return String[] than an Object array
		String[] recipientIDs = userIds.toArray(new String[0]);
        
		return recipientIDs;
	}

	/**
	 * Call notification Engine passing an event object to it
	 * @param event
	 * @throws NotificationException
	 */
	private void sendNotification(NotificationEvent event) throws NotificationException {
		try {
			//Call notify method of NotificationService to pass on the event 
			//to notification engine
			NotificationService notificationService = Platform.getService(
				NotificationService.class);
			notificationService.notify(event);
		} catch (EventException e) {
			throw new NotificationException(e.getMessage(), e.getCause());
		} catch (UnresolvedNotificationDataException e) {
			throw new NotificationException(e.getMessage(), e.getCause());
		} catch (TemplateNotFoundException e) {
			throw new NotificationException(e.getMessage(), e.getCause());
		} catch (MultipleTemplateException e) {
			throw new NotificationException(e.getMessage(), e.getCause());
		} catch (NotificationResolverNotFoundException e) {
			throw new NotificationException(e.getMessage(), e.getCause());
		} catch (UserDetailsNotFoundException e) {
			throw new NotificationException(e.getMessage(), e.getCause());
		} catch (NotificationException e){
			throw e;
		}
	}	
}

